param_from = {"from_"}
param_target = {"target_id"}
param_name = {"faction_name"}
param_stat = {"stat_key", "stat_keys"}
param_log = {"attack_log_id", "log_id"}
param_ids = {"category_ids", "honor_ids", "item_ids", "medals_ids", "territory_ids"}
param_filters = {"attack_filters", "crimes_filter", "property_filters"}
param_id = {
    "chain_id",
    "crime_id",
    "faction_id",
    "item_id",
    "listing_id",
    "log_category_id_torn",
    "property_id",
    "property_type_id",
    "race_id",
    "raid_war_id",
    "ranked_war_id",
    "team_id",
    "territory_war_id",
    "thread_id",
    "track_id",
    "user_id",
}
param_cat = {
    "balance_category",
    "bazaar_category",
    "car_category",
    "contributors_category",
    "crimes_category",
    "item_category",
    "leaderboard_category",
    "list_category",
    "log_category_id",
    "news_category",
    "race_category",
    "report_category",
    "stat_category",
    "warfare_category",
}
param_group = {
    "from": param_from,
    "target": param_target,
    "name": param_name,
    "stat": param_stat,
    "log": param_log,
    "ids": param_ids,
    "filters": param_filters,
    "id": param_id,
    "cat": param_cat,
}
parameter_map = {key: group for group, keys in param_group.items() for key in keys}


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..params.builders import build_params
from ..type_hints import SortOptions, MrktBazaarCatOptions, MrktItemBonusOptions

if TYPE_CHECKING:
    from ..client import TornAPIWrapper

class Market:
    """
    Market API endpoints.
    """

    def __init__(self, api: TornAPIWrapper):
        self.api = api

    def get_auctionhouselisting(self, listing_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get specific item auction house listings.
        API key (Public).
        :param listing_id: Listing id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/auctionhouselisting", build_params(self.get_auctionhouselisting, locals()))

    def get_auctionhouse(self, item_id: int = None, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get auction house listings or specific item auction house listings.
        API key (Public).
        :param item_id: Item id.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/auctionhouse", build_params(self.get_auctionhouse, locals()))

    def get_bazaar(self, item_id: int = None, bazaar_category: MrktBazaarCatOptions = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get bazaar directory or specialized bazaar directory.
        API key (Public).
        The default response is of type 'BazaarWeekly', but if a category is chosen, the response will be of type 'BazaarSpecialized'.
        :param item_id: Item id.
        :param bazaar_category: Category of specialized bazaars returned.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/bazaar", build_params(self.get_bazaar, locals()))

    def get_itemmarket(self, item_id: int, bonus: MrktItemBonusOptions = None, limit: int = 20, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get item market listings.
        API key (Public).
        :param item_id: Item id.
        :param bonus: Used to filter weapons with a specific bonus.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/itemmarket", build_params(self.get_itemmarket, locals()))

    def get_properties(self, property_type_id: int, limit: int = 20, offset: int = 0, sort: SortOptions = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get properties market listings.
        API key (Public).
        :param property_type_id: Property type id.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param sort: Sorted by the greatest timestamps.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/properties", build_params(self.get_properties, locals()))

    def get_rentals(self, property_type_id: int, limit: int = 20, offset: int = 0, sort: SortOptions = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get properties rental listings.
        API key (Public).
        :param property_type_id: Property type id.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param sort: Sorted by the greatest timestamps.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/rentals", build_params(self.get_rentals, locals()))

    def get_lookup(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all available market selections.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/lookup", build_params(self.get_lookup, locals()))

    def get_timestamp(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get current server time.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/market/timestamp", build_params(self.get_timestamp, locals()))


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..params.builders import build_params
from ..type_hints import SortOptions, RaceCatOptions, RaceCarCatOptions

if TYPE_CHECKING:
    from ..client import TornAPIWrapper

class Racing:
    """
    Racing API endpoints.
    """

    def __init__(self, api: TornAPIWrapper):
        self.api = api

    def get_cars(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get cars and their racing stats.
        API key (Public).
        Returns the stat details about racing cars.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/cars", build_params(self.get_cars, locals()))

    def get_carupgrades(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all possible car upgrades.
        API key (Public).
        Returns the details about all possible car upgrades.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/carupgrades", build_params(self.get_carupgrades, locals()))

    def get_races(self, race_category: RaceCatOptions = None, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get races.
        API key (Public).
        Returns a list of races, ordered by race start timestamp.
        :param race_category: Category of races returned.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned.
        :param from_: Timestamp that sets the lower limit for the data returned.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/races", build_params(self.get_races, locals()))

    def get_race(self, race_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get specific race details.
        API key (Public).
        Returns the details of a race.
        :param race_id: Race id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/race", build_params(self.get_race, locals()))

    def get_records(self, track_id: int, car_category: RaceCarCatOptions, timestamp: int = None, comment: str = None) -> dict:
        """
        Get track records.
        API key (Public).
        Returns a list of 5 best lap records for the chosen track and car class.
        :param track_id: Track id.
        :param car_category: Car class.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/records", build_params(self.get_records, locals()))

    def get_tracks(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get race tracks and descriptions.
        API key (Public).
        Returns the details about racing tracks.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/tracks", build_params(self.get_tracks, locals()))

    def get_lookup(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all available racing selections.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/lookup", build_params(self.get_lookup, locals()))

    def get_timestamp(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get current server time.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/racing/timestamp", build_params(self.get_timestamp, locals()))



"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..params.builders import build_params

if TYPE_CHECKING:
    from ..client import TornAPIWrapper

class Property:
    """
    Property API endpoints.
    """

    def __init__(self, api: TornAPIWrapper):
        self.api = api

    def get_property(self, property_id: int, timestamp: int = None, comment: str = None):
        """
        Get a specific property.
        API key (Public).
        :param property_id: Property id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/property/property", build_params(self.get_property, locals()))

    def get_lookup(self, timestamp: int = None, comment: str = None):
        """
        Get all available property selections.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/property/lookup", build_params(self.get_lookup, locals()))

    def get_timestamp(self, timestamp: int = None, comment: str = None):
        """
        Get current server time.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/property/timestamp", build_params(self.get_timestamp, locals()))


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..params.builders import build_params
from ..type_hints import SortOptions, RaceCatOptions, UserListCatOptions, AttackFiltersOptions, \
    UserPropertiesFiltersOptions, ReportCatOptions, UserPrsnlStatsCatOptions, UserPrsnlStatsStatOptions

if TYPE_CHECKING:
    from ..client_async import TornAPIWrapperAsync

class User:
    """
    User API endpoints.
    """

    def __init__(self, api: TornAPIWrapperAsync):
        self.api = api

    async def get_ammo(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your ammo information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/ammo", build_params(self.get_ammo, locals()))

    async def get_attacks(self, attack_filters: list[AttackFiltersOptions] = None, limit: int = 100, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your detailed attacks.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/attacks", build_params(self.get_attacks, locals()))

    async def get_attacksfull(self, attack_filters: list[AttackFiltersOptions] = None, limit: int = 1000, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your simplified attacks.
        API key (Limited).
        Returns up to 1,000 rows.
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/attacksfull", build_params(self.get_attacksfull, locals()))

    async def get_bars(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your bars information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/bars", build_params(self.get_bars, locals()))

    async def get_basic(self, user_id: int = None, striptags: bool = True, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your basic profile information or for a specific user.
        API key (Public).
        :param user_id: User id or user discord id.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/basic", build_params(self.get_basic, locals()))

    async def get_battlestats(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your battlestats.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/battlestats", build_params(self.get_battlestats, locals()))

    async def get_bounties(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get bounties placed on you or for a specific user.
        API key (Public).
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/bounties", build_params(self.get_bounties, locals()))

    async def get_calendar(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your calendar events start time.
        API key (Minimal).
        Only available to yourself.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/calendar", build_params(self.get_calendar, locals()))

    async def get_competition(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your competition information or for a specific player.
        API key (Public).
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/competition", build_params(self.get_competition, locals()))

    async def get_cooldowns(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your cooldowns information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/cooldowns", build_params(self.get_cooldowns, locals()))

    async def get_crimes(self, crime_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your crime statistics.
        API key (Minimal).
        Return the details and statistics about for a specific crime.
        :param crime_id: Crime id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/crimes", build_params(self.get_crimes, locals()))

    async def get_discord(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your discord information or for a specific user.
        API key (Public).
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/discord", build_params(self.get_discord, locals()))

    async def get_education(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your education information.
        API key (Public).
        The response contains a list of complete eduactions and of a current education (if any).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/education", build_params(self.get_education, locals()))

    async def get_enlistedcars(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your enlisted cars.
        API key (Minimal).
        Returns a list of all user enlisted cars.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/enlistedcars", build_params(self.get_enlistedcars, locals()))

    async def get_equipment(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your equipment & clothing.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/equipment", build_params(self.get_equipment, locals()))

    async def get_events(self, striptags: bool = False, limit: int = 20, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your events.
        API key (Limited).
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/events", build_params(self.get_events, locals()))

    async def get_faction(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction information or for a specific player.
        API key (Public).
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/faction", build_params(self.get_faction, locals()))

    async def get_forumfeed(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get updates on your threads and posts.
        API key (Minimal).
        This selection returns data visible in 'Feed' section on forum page. Feed is sorted by timestamp descending. Only a maximum of 100 rows are returned.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/forumfeed", build_params(self.get_forumfeed, locals()))

    async def get_forumfriends(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get updates on your friends' activity.
        API key (Minimal).
        This selection returns data visible in 'Friends' section on forum page. Feed is sorted by timestamp descending. Only a maximum of 100 rows are returned.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/forumfriends", build_params(self.get_forumfriends, locals()))

    async def get_forumposts(self, user_id: int = None, striptags: bool = True, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your posts or for a specific player.
        API key (Public).
        Returns 20 posts per page.
        :param user_id: User id or user discord id.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/forumposts", build_params(self.get_forumposts, locals()))

    async def get_forumsubscribedthreads(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get updates on threads you subscribed to.
        API key (Minimal).
        This selection returns data visible in 'Subscribed Threads' section on forum page. Threads are sorted in the same way as on site.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/forumsubscribedthreads", build_params(self.get_forumsubscribedthreads, locals()))

    async def get_forumthreads(self, user_id: int = None, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your threads or for a specific player.
        API key (Public).
        Returns up to 100 threads per page. The field 'new_posts' is also available, indicating the amount of unread posts with a Minimum API key (or higher).
        Specific Player - When requesting data for the key owner, a field 'new_posts' is also available, indicating the amount of unread posts. Minimum API key is required for that.
        :param user_id: User id or user discord id.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/forumthreads", build_params(self.get_forumthreads, locals()))

    async def get_hof(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your hall of fame rankings or for a specific player.
        API key (Public).
        When requesting selection with Limited, Full or Custom key, battle_stats selection will be populated.
        Specific Player - The battle_stats selection will be populated only when requesting selection with Limited, Full or Custom key and for yourself.
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/hof", build_params(self.get_hof, locals()))

    async def get_honors(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your achieved honors.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/honors", build_params(self.get_honors, locals()))

    async def get_icons(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your icons information or for a specific player.
        API key (Public).
        When requesting data for yourself with 'Custom', 'Limited' or 'Full' access keys, the response will be of type UserIconPrivate, otherwise UserIconPublic.
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/icons", build_params(self.get_icons, locals()))

    async def get_itemmarket(self, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your item market listings for a specific item.
        API key (Limited).
        :param offset: Number of rows to skip before returning results.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/itemmarket", build_params(self.get_itemmarket, locals()))

    async def get_job(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your job information or for a specific player.
        API key (Public).
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/job", build_params(self.get_job, locals()))

    async def get_jobpoints(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your jobpoints.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/jobpoints", build_params(self.get_jobpoints, locals()))

    async def get_jobranks(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your starter job positions.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/jobranks", build_params(self.get_jobranks, locals()))

    async def get_list(self, list_category: UserListCatOptions, striptags: bool = True, limit: int = 50, offset: int = None, sort: SortOptions = "ASC", timestamp: int = None, comment: str = None) -> dict:
        """
        Get your friends, enemies or targets list.
        API key (Limited).
        :param list_category: Select list type.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param sort: Sort rows from newest to oldest.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/list", build_params(self.get_list, locals()))

    async def get_log(self, log_id: list[int] = None, log_category_id: int = None, target_id: int = None, limit: int = 20, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your logs.
        API key (Full).
        It's possible to pass a list of log ids or a log category id.
        :param log_id: Log ids.
        :param log_category_id: Log category id.
        :param target_id: Get logs where you interacted with a specific player by passing their player ID.
        :param limit: Number of results to return.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/log", build_params(self.get_log, locals()))

    async def get_medals(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your achieved medals.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/medals", build_params(self.get_medals, locals()))

    async def get_merits(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your merits.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/merits", build_params(self.get_merits, locals()))

    async def get_messages(self, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your messages.
        API key (Limited).
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/messages", build_params(self.get_messages, locals()))

    async def get_missions(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your current missions information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/missions", build_params(self.get_missions, locals()))

    async def get_money(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your current wealth.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/money", build_params(self.get_money, locals()))

    async def get_newevents(self, striptags: bool = False, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your unseen events.
        API key (Limited).
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/newevents", build_params(self.get_newevents, locals()))

    async def get_newmessages(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your unseen messages.
        API key (Limited).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/newmessages", build_params(self.get_newmessages, locals()))

    async def get_notifications(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your notifications.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/notifications", build_params(self.get_notifications, locals()))

    async def get_organizedcrime(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your current ongoing organized crime.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/organizedcrime", build_params(self.get_organizedcrime, locals()))

    async def get_organizedcrimes(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get available slots for organized crimes with status 'Recruiting'.
        API key (Minimal).
        Unlike 'faction' -> 'crimes', this selection only shows empty slots, and only for crimes with the 'Recruiting' status.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/organizedcrimes", build_params(self.get_organizedcrimes, locals()))

    async def get_personalstats(self, user_id: int = None, stat_category: UserPrsnlStatsCatOptions = None, stat_keys: list[UserPrsnlStatsStatOptions] = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your personal stats or for a specific user.
        API key (Public).
         - UserPersonalStatsFull is returned only when this selection is requested with Limited, Full or Custom key access key.
         - UserPersonalStatsFullPublic is returned when the requested category is 'all'.
         - UserPersonalStatsPopular is returned when the requested category is 'popular'. Please try to use UserPersonalStatsPopular over UserPersonalStatsFullPublic wherever possible in order to reduce the server load.
         - Otherwise, UserPersonalStatsCategory is returned for the matched category.
         - Historical stats can be fetched via 'stat' query parameter. It's only possible to pass up to 10 historical stats at once (the rest is trimmed). When requesting historical stats the response will be of type UserPersonalStatsHistoric.
         - Use 'timestamp' query parameter to get historical stats at the certain point in time. It's possible some historical stats didn't exist at the given timestamp, and for such stats you will not receive anything back.
        Specific Player:
         - It's possible to request specific stats via 'stat' parameter. In this case the response will vary depending on the stats requested. Private stats are still available only to the key owner (with Limited or higher key).
         - Additionally, historical stats can also be fetched via 'stat' query parameter, but 'timestamp' parameter must be provided as well. It's only possible to pass up to 10 historical stats at once (the rest is trimmed). When requesting historical stats the response will be of type UserPersonalStatsHistoric.
        :param user_id: User id or user discord id.
        :param stat_category: Personal stat category.
        :param stat_keys: Stat names (10 maximum). Used to fetch historical stat values.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/personalstats", build_params(self.get_personalstats, locals()))

    async def get_profile(self, user_id: int = None, striptags: bool = True, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your profile information or for a specific player.
        API key (Public).
        :param user_id: User id or user discord id.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/profile", build_params(self.get_profile, locals()))

    async def get_properties(self, user_id: int = None, property_filters: UserPropertiesFiltersOptions = None, limit: int = 20, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your own properties or for a specific user.
        API key (Public).
        Extended responses are available when requesting the data with Limited or higher access keys.
        Specific Player - Extended responses are available when requesting the data with Limited or higher access keys for yourself or your spouse.
        :param user_id: User id or user discord id.
        :param property_filters: It's possible to use this query parameter to filter properties by the key owner or their spouse.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/properties", build_params(self.get_properties, locals()))

    async def get_property(self, user_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your current property or for a specific user.
        API key (Public).
        :param user_id: User id or user discord id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/property", build_params(self.get_property, locals()))

    async def get_races(self, race_category: RaceCatOptions = None, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get user races.
        API key (Minimal).
        :param race_category: Category of races returned.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/races", build_params(self.get_races, locals()))

    async def get_racingrecords(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your current racing records.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/racingrecords", build_params(self.get_racingrecords, locals()))

    async def get_refills(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your refills information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/refills", build_params(self.get_refills, locals()))

    async def get_reports(self, report_category: ReportCatOptions = None, target_id: int = None, limit: int = 20, offset: int = 0, sort: SortOptions = "DESC", timestamp: int = None, comment: str = None) -> dict:
        """
        Get your reports.
        API key (Limited).
        :param report_category: Used to filter reports with a specific type.
        :param target_id: Get reports for a specific player by passing their player ID.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param sort: Sorted by the greatest timestamps.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/reports", build_params(self.get_reports, locals()))

    async def get_revives(self, attack_filters: list[AttackFiltersOptions] = None, striptags: bool = True, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your detailed revives.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/revives", build_params(self.get_revives, locals()))

    async def get_revivesfull(self, attack_filters: list[AttackFiltersOptions] = None, striptags: bool = True, limit: int = 20, sort: SortOptions = None, to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your simplified revives.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/revivesFull", build_params(self.get_revivesfull, locals()))

    async def get_skills(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your skills.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/skills", build_params(self.get_skills, locals()))

    async def get_travel(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your travel information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/travel", build_params(self.get_travel, locals()))

    async def get_virus(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your virus information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/virus", build_params(self.get_virus, locals()))

    async def get_weaponexp(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your weapon experience information.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/weaponexp", build_params(self.get_weaponexp, locals()))

    async def get_workstats(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your working stats.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/workstats", build_params(self.get_workstats, locals()))

    async def get_lookup(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all available user selections.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/lookup", build_params(self.get_lookup, locals()))

    async def get_timestamp(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get current server time.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return await self.api.request("/user/timestamp", build_params(self.get_timestamp, locals()))


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..params.builders import build_params
from ..type_hints import SortOptions, TornFactionHofCatOptions, TornHofCatOptions, TornItemsCatOptions

if TYPE_CHECKING:
    from ..client import TornAPIWrapper

class Torn:
    """
    Torn API endpoints.
    """
    def __init__(self, api: TornAPIWrapper):
        self.api = api

    def get_attacklog(self, attack_log_id: str, striptags: bool = True, offset: int = 0, sort: SortOptions = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get attack log details.
        API key (Public).
        :param attack_log_id: Code of the attack log. E.g. d51ad4fe6be884b309b142e2d1d4f807.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param offset: Offset used for paging.
        :param sort: Sorted by the greatest timestamps.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/attacklog", build_params(self.get_attacklog, locals()))

    def get_bounties(self, limit: int = 100, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get bounties.
        API key (Public).
        :param limit: Number of results to return.
        :param offset: Offset used for paging.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/bounties", build_params(self.get_bounties, locals()))

    def get_calendar(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get calendar information.
        API key (Public).
        Get the details about competitions & events in the running year.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/calendar", build_params(self.get_calendar, locals()))

    def get_crimes(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get crimes information.
        API key (Public).
        Return the details about released crimes.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/crimes", build_params(self.get_crimes, locals()))

    def get_education(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get education information.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/education", build_params(self.get_education, locals()))

    def get_elimination(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get current standings for all elimination teams.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/elimination", build_params(self.get_elimination, locals()))

    def get_eliminationteam(self, team_id: int, limit: int = 100, offset: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get players in a specific elimination team.
        API key (Public).
        :param team_id: Elimination team id.
        :param limit: Number of results to return.
        :param offset: Offset used for paging.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/eliminationteam", build_params(self.get_eliminationteam, locals()))

    def get_factionhof(self, leaderboard_category: TornFactionHofCatOptions, limit: int = 100, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get faction hall of fame positions for a specific category.
        API key (Public).
        :param leaderboard_category: Leaderboards category.
        :param limit: Number of results to return.
        :param offset: Offset used for paging.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/factionhof", build_params(self.get_factionhof, locals()))

    def get_factiontree(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get full faction tree.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/factiontree", build_params(self.get_factiontree, locals()))

    def get_honors(self, honor_ids: list[int] = None, limit: int = 20, offset: int = 0, sort: SortOptions = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all honors or specific honors.
        API key (Public).
        :param honor_ids: Honor id or a list of honor ids.
        :param limit: Number of results to return.
        :param offset: Offset used for paging.
        :param sort: Sorted by the greatest timestamps.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/honors", build_params(self.get_honors, locals()))

    def get_hof(self, leaderboard_category: TornHofCatOptions, limit: int = 100, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get player hall of fame positions for a specific category.
        API key (Public).
        :param leaderboard_category: Leaderboards category.
        :param limit: Number of results to return.
        :param offset: Offset used for paging.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/hof", build_params(self.get_hof, locals()))

    def get_itemammo(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get information about ammo.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/itemammo", build_params(self.get_itemammo, locals()))

    def get_itemdetails(self, item_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get information about a specific item.
        API key (Public).
        :param item_id: Item uid.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/itemdetails", build_params(self.get_itemdetails, locals()))

    def get_itemmods(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get information about weapon upgrades.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/itemmods", build_params(self.get_itemmods, locals()))

    def get_items(self, item_ids: list[int] = None, item_category: TornItemsCatOptions = None, sort: SortOptions = "ASC", timestamp: int = None, comment: str = None) -> dict:
        """
        Get information about items.
        API key (Public).
        Default category is 'All'.
        Details are not populated when requesting the category 'All'.
        Specific Items - Details are always populated when available.
        :param item_ids: Item id or a list of item ids.
        :param item_category: Item category type.
        :param sort: Sort rows from newest to oldest. Default ordering is ascending.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/items", build_params(self.get_items, locals()))

    def get_logcategories(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get available log categories.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/logcategories", build_params(self.get_logcategories, locals()))

    def get_logtypes(self, log_category_id_torn: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get available log categories or available log ids for a specific log category.
        API key (Public).
        :param log_category_id_torn: Log category id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/logtypes", build_params(self.get_logtypes, locals()))

    def get_medals(self, medals_ids: list[int] = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all medals or specific medals.
        API key (Public).
        :param medals_ids: Medal id or a list of medal ids.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/medals", build_params(self.get_medals, locals()))

    def get_merits(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all merits.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/merits", build_params(self.get_merits, locals()))

    def get_organizedcrimes(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get organized crimes information.
        API key (Public).
        Return the details about released faction organized crimes.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/organizedcrimes", build_params(self.get_organizedcrimes, locals()))

    def get_properties(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get properties details.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/properties", build_params(self.get_properties, locals()))

    def get_subcrimes(self, crime_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get Subcrimes information.
        API key (Public).
        Return the details about possible actions for a specific crime.
        :param crime_id: Crime id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/subcrimes", build_params(self.get_subcrimes, locals()))

    def get_territory(self, territory_ids: list[str] = None, limit: int = 20, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get territory details.
        API key (Public).
        :param territory_ids: Specific territory id or a list of territory ids.
        :param limit: Number of results to return.
        :param offset: Offset used for paging.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/territory", build_params(self.get_territory, locals()))

    def get_lookup(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all available torn selections.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/lookup", build_params(self.get_lookup, locals()))

    def get_timestamp(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get current server time.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/torn/timestamp", build_params(self.get_timestamp, locals()))


import inspect
from typing import Callable

from ..params.map import parameter_map

def build_local_params(method_request: Callable, local_vars: dict) -> dict | None:
    """
    Extracts the arguments passed to `method_request` and returns them as a clean dictionary ready for further processing.
    :param method_request: The `TornAPIWrapper` method.
    :param local_vars: A dictionary containing the `method_request` scope's local variables.
    :return: A sterilized dictionary for further processing | None.
    :rtype: dict | None
    """
    return {param: local_vars[param] for param in inspect.signature(method_request).parameters if local_vars[param] is not None and param != "self"} or None

def build_params(method_request: Callable, local_vars: dict) -> dict | None:
    """
    Builds Torn API compatible query parameters from wrapper method arguments.
    :param method_request: The `TornAPIWrapper` method.
    :param local_vars: A dictionary containing the `method_request` scope's local variables.
    :return: A formatted dictionary of API query parameters | None.
    :rtype: dict | None
    """
    local_params = build_local_params(method_request, local_vars)
    if local_params is None:
        return None
    api_params = {}
    for param, value in local_params.items():
        if isinstance(value, list) and value: #Format `array[string]` or `array[int]` query for the API e.g. `get_attacks()`.
            value = ",".join(map(str, value))
        elif param == "timestamp" and isinstance(value, int): #Format Python timestamp[int] to API friendly timestamp[str].
            value = str(value)
        elif param == "striptags" and isinstance(value, bool): #Format Python striptags[bool] to API friendly striptags[str].
            value = str(value).lower()
        api_param_var = parameter_map.get(param, param) #Replace param such as `stat_category` with `cat` for the API. The wrapper param is not `cat` since type hinting is not clear to the user.
        api_params[api_param_var] = value
    return api_params or None


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

#Torn Wrapper Errors
class WrapperError(Exception):
    """
    Base class for all TornAPIWrapper errors.
    """
    pass

class UnhandledAPIError(WrapperError):
    """
    An API error code was received that is not yet implemented by TornAPIWrapper.
    Please open an issue at https://github.com/cxdzc/TornAPIWrapper/issues/new/choose or
    contact AfricanChild [3157295], including details on how you encountered this error.
    """
    pass

#Torn API Errors
class APIError(Exception):
    """
    Base class for all API-related errors.
    """
    def __init__(self, message: str, code: int):
        super().__init__(message)
        self.code = code

class UnknownError(APIError):
    """
    0
    Unknown error
    Unhandled error, should not occur.
    """
    pass

class EmptyKey(APIError):
    """
    1
    Key is empty
    Private key is empty in current request.
    """
    pass

class InvalidKey(APIError):
    """
    2
    Incorrect Key
    Private key is wrong/incorrect format.
    """
    pass

class WrongType(APIError):
    """
    3
    Wrong type
    Requesting an incorrect basic type.
    """
    pass

class WrongFields(APIError):
    """
    4
    Wrong fields
    Requesting incorrect selection fields.
    """
    pass

class RateLimit(APIError):
    """
    5
    Too many requests
    Requests are blocked for a small period of time because of too many requests per user (max 100 per minute).
    """
    pass

class InvalidId(APIError):
    """
    6
    Incorrect ID
    Wrong ID value.
    """
    pass

class InvalidRelation(APIError):
    """
    7
    Incorrect ID-entity relation
    A requested selection is private (For example, personal data of another user / faction).
    """
    pass

class IpBlocked(APIError):
    """
    8
    IP block
    Current IP is banned for a small period of time because of abuse.
    """
    pass

class ApiDisabled(APIError):
    """
    9
    API disabled
    Api system is currently disabled.
    """
    pass

class KeyOwnerJailed(APIError):
    """
    10
    Key owner is in federal jail
    Current key can't be used because owner is in federal jail.
    """
    pass

class KeyChangeLimit(APIError):
    """
    11
    Key change error
    You can only change your API key once every 60 seconds.
    """
    pass

class KeyReadFailed(APIError):
    """
    12
    Key read error
    Error reading key from Database.
    """
    pass

class KeyInactive(APIError):
    """
    13
    The key is temporarily disabled due to owner inactivity
    The key owner hasn't been online for more than 7 days.
    """
    pass

class DailyLimit(APIError):
    """
    14
    Daily read limit reached
    Too many records have been pulled today by this user from our cloud services.
    """
    pass

class TemporaryFailure(APIError):
    """
    15
    Temporary error
    An error code specifically for testing purposes that has no dedicated meaning.
    """
    pass

class PermissionDenied(APIError):
    """
    16
    Access level of this key is not high enough
    A selection is being called of which this key does not have permission to access.
    """
    pass

class BackendFailure(APIError):
    """
    17
    Backend error occurred, please try again.
    """
    pass

class KeyPaused(APIError):
    """
    18
    API key has been paused by the owner.
    """
    pass

class MigrationRequired(APIError):
    """
    19
    Must be migrated to crimes 2.0.
    """
    pass

class RaceNotFinished(APIError):
    """
    20
    Race not yet finished.
    """
    pass

class InvalidCategory(APIError):
    """
    21
    Incorrect category
    Wrong cat value.
    """
    pass

class ApiV1Only(APIError):
    """
    22
    This selection is only available in API v1.
    """
    pass

class ApiV2Only(APIError):
    """
    23
    This selection is only available in API v2.
    """
    pass

class TemporarilyClosed(APIError):
    """
    24
    Closed temporarily.
    """
    pass

class InvalidStat(APIError):
    """
    25
    Invalid stat requested.
    """
    pass

class InvalidParameterCombination(APIError):
    """
    26
    Only category or stats can be requested
    """
    pass

class MigrationRequiredOC2(APIError):
    """
    27
    Must be migrated to organized crimes 2.0.
    """
    pass


class InvalidLogId(APIError):
    """
    28
    Incorrect log ID.
    """
    pass


class InvalidInteractionLogCategory(APIError):
    """
    29
    Category selection is not available for interaction logs.
    """
    pass

class TornAPIErrorHandler:
    """
    Handle Torn API error codes and raise appropriate exceptions.
    """
    api_error_codes = {
        0: (UnknownError, "Unknown error - Unhandled error, should not occur."),
        1: (EmptyKey, "Key is empty - Private key is empty in current request."),
        2: (InvalidKey, "Incorrect Key - Private key is wrong/incorrect format."),
        3: (WrongType, "Wrong type - Requesting an incorrect basic type."),
        4: (WrongFields, "Wrong fields - Requesting incorrect selection fields."),
        5: (RateLimit, "Too many requests - Requests are blocked for a small period of time because of too many requests per user (max 100 per minute)."),
        6: (InvalidId, "Incorrect ID - Wrong ID value."),
        7: (InvalidRelation, "Incorrect ID-entity relation - A requested selection is private (For example, personal data of another user / faction)."),
        8: (IpBlocked, "IP block - Current IP is banned for a small period of time because of abuse."),
        9: (ApiDisabled, "API disabled - Api system is currently disabled."),
        10: (KeyOwnerJailed, "Key owner is in federal jail - Current key can't be used because owner is in federal jail."),
        11: (KeyChangeLimit, "Key change error - You can only change your API key once every 60 seconds."),
        12: (KeyReadFailed, "Key read error - Error reading key from Database."),
        13: (KeyInactive, "The key is temporarily disabled due to owner inactivity - The key owner hasn't been online for more than 7 days."),
        14: (DailyLimit, "Daily read limit reached - Too many records have been pulled today by this user from our cloud services."),
        15: (TemporaryFailure, "Temporary error - An error code specifically for testing purposes that has no dedicated meaning."),
        16: (PermissionDenied, "Access level of this key is not high enough - A selection is being called of which this key does not have permission to access."),
        17: (BackendFailure, "Backend error occurred, please try again."),
        18: (KeyPaused, "API key has been paused by the owner."),
        19: (MigrationRequired, "Must be migrated to crimes 2.0."),
        20: (RaceNotFinished, "Race not yet finished."),
        21: (InvalidCategory, "Incorrect category - Wrong cat value."),
        22: (ApiV1Only, "This selection is only available in API v1."),
        23: (ApiV2Only, "This selection is only available in API v2."),
        24: (TemporarilyClosed, "Closed temporarily."),
        25: (InvalidStat, "Invalid stat requested."),
        26: (InvalidParameterCombination, "Only category or stats can be requested"),
        27: (MigrationRequiredOC2, "Must be migrated to organized crimes 2.0."),
        28: (InvalidLogId, "Incorrect log ID."),
        29: (InvalidInteractionLogCategory, "Category selection is not available for interaction logs."),
    }

    def raise_code(self, error_code: int):
        """
        Raise an exception based on a Torn API error code.
        :param error_code: API error code returned by Torn.
        :raises UnhandledAPIError: If the error code is not implemented.
        :raises TornAPIError: If the error code is recognized.
        :rtype: None
        """
        error = self.api_error_codes.get(error_code)
        if error is None:
            raise UnhandledAPIError("An API error code was received that is not yet implemented by TornAPIWrapper.\nPlease open an issue at https://github.com/cxdzc/TornAPIWrapper/issues/new/choose or contact AfricanChild [3157295], including details on how you encountered this error.")
        api_error_class, message = error
        raise api_error_class(message=message, code=error_code)


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..params.builders import build_params
from ..type_hints import SortOptions, FacCatScopeOptions, AttackFiltersOptions, FacContributorsStatOptions, \
    FacCrimesCatOptions, FacCrimesFiltersOptions, FacNewsCatOptions, ReportCatOptions, FacWarfareCatOptions

if TYPE_CHECKING:
    from ..client import TornAPIWrapper

class Faction:
    """
    Faction API endpoints.
    """

    def __init__(self, api: TornAPIWrapper):
        self.api = api

    def get_applications(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's applications.
        Requires faction API access permissions.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/applications", build_params(self.get_applications, locals()))

    def get_attacks(self, attack_filters: list[AttackFiltersOptions] = None, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's detailed attacks.
        Requires faction API access permissions.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/attacks", build_params(self.get_attacks, locals()))

    def get_attacksfull(self, attack_filters: list[AttackFiltersOptions] = None, limit: int = 1000, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's simplified attacks
        Requires faction API access permissions.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/attacksfull", build_params(self.get_attacksfull, locals()))

    def get_balance(self, balance_category: FacCatScopeOptions = "current", timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's & member's balance details.
        Requires faction API access permissions.
        API key (Limited).
        :param balance_category: By default, this selection will return only current faction's member balances, and the option 'all' will return all current members balances + additionally those of ex-members which do have money or points on their balance.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/balance", build_params(self.get_balance, locals()))

    def get_basic(self, faction_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's basic details or a faction's basic details.
        API key (Public).
        The 'is_enlisted' value will be populated if you have API faction permissions (with custom, limited or full access keys), otherwise it will be set as null.
        Specific Faction - The 'is_enlisted' value will be populated if you're requesting data for your faction and have faction permissions (with custom, limited or full access keys), otherwise it will be set as null.
        :param faction_id: Faction id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/basic", build_params(self.get_basic, locals()))

    def get_chain(self, faction_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's current chain or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/chain", build_params(self.get_chain, locals()))

    def get_chains(self, faction_id: int = None, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get a list of completed chains for your faction or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/chains", build_params(self.get_chains, locals()))

    def get_chainreport(self, chain_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get the latest chain report for your faction or a specific faction.
        API key (Public).
        This includes currently ongoing chains.
        Specifc Faction - Chain reports for ongoing chains are available only for your own faction.
        :param chain_id: Chain id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/chainreport", build_params(self.get_chainreport, locals()))

    def get_contributors(self, stat_key: FacContributorsStatOptions, contributors_category: FacCatScopeOptions = "current", timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's challenge contributors.
        Requires faction API access permissions.
        API key (Public).
        :param stat_key: Get contributors for this field.
        :param contributors_category: By default, this selection will return only current faction's member contributions, and the option 'all' will return all contributors.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/contributors", build_params(self.get_contributors, locals()))

    def get_crimes(self, crimes_category: FacCrimesCatOptions = "all", crimes_filter: FacCrimesFiltersOptions = None, offset: int = 0, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's organized crimes.
        Requires faction API access permissions.
        API key (Minimal).
        It's possible to get older entries either by timestamp range (from, to) or with offset.
        Crimes are ordered depending on the category chosen:
         - For categories 'all' & 'available', the ordering field is 'created_at'.
         - For categories 'successful', 'failed' & 'completed', the ordering field is 'executed_at'.
         - For categories 'recruiting' & 'expired', the ordering field is 'expired_at'.
         - For category 'planning', the ordering field is 'ready_at'.
        :param crimes_category: Category of organized crimes returned. Category 'available' includes both 'recruiting' & 'planning', and category 'completed' includes both 'successful' & 'failure'.
        :param crimes_filter: It's possible to set this parameter to specify a field used for the sort, from & to query parameters. If not specified, the field will default to the category sorting as described above.
        :param offset: Number of rows to skip before returning results.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/crimes", build_params(self.get_crimes, locals()))

    def get_crime(self, crime_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get a specific organized crime.
        API key (Minimal).
        :param crime_id: Crime id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/crime", build_params(self.get_crime, locals()))

    def get_hof(self, faction_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get the hall of fame rankings for your faction or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/hof", build_params(self.get_hof, locals()))

    def get_members(self, faction_id: int = None, striptags: bool = True, timestamp: int = None, comment: str = None) -> dict:
        """
        Get a list of members for your faction or a specific faction.
        API key (Public).
        The 'revive_setting' value will be populated (not Unknown) if you have faction permissions (with custom, limited or full access keys), otherwise it will be set as 'Unknown'.
        Specific Faction - The 'revive_setting' value will be populated (not Unknown) if you're requesting data for your own faction and have faction permissions (with custom, limited or full access keys), otherwise it will be set as 'Unknown'.
        :param faction_id: Faction id.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/members", build_params(self.get_members, locals()))

    def get_news(self, news_category: FacNewsCatOptions, striptags: bool = False, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's news details.
        Requires faction API access permissions.
        API key (Minimal).
        It is possible to pass up to 10 categories at the time.
        :param news_category: News category type.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/news", build_params(self.get_news, locals()))

    def get_positions(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's positions details.
        Requires faction API access permissions.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/positions", build_params(self.get_positions, locals()))

    def get_rackets(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get a list of current rackets.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/rackets", build_params(self.get_rackets, locals()))

    def get_raidreport(self, raid_war_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get raid war details.
        API key (Public).
        :param raid_war_id: Raid war id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/raidreport", build_params(self.get_raidreport, locals()))

    def get_raids(self, faction_id: int = None, limit: int = 20, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get the raids history for your faction or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/raids", build_params(self.get_raids, locals()))

    def get_rankedwars(self, faction_id: int = None, limit: int = 20, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get the ranked wars history for your faction or a specific faction.
        Requires faction API access permissions.
        API key (Public).
        Use offset to get older results which are always ordered descending.
        :param faction_id: Faction id.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/rankedwars", build_params(self.get_rankedwars, locals()))

    def get_rankedwarreport(self, ranked_war_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get ranked war details.
        API key (Public).
        :param ranked_war_id: Ranked war id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/rankedwarreport", build_params(self.get_rankedwarreport, locals()))

    def get_reports(self, report_category: ReportCatOptions = None, target_id: int = None, limit: int = 20, offset: int = 0, sort: SortOptions = "DESC", timestamp: int = None, comment: str = None) -> dict:
        """
        Get faction reports.
        API key (Limited).
        The default limit is set to 25. However, the limit can be set to 100 for the 'stats' category.
        :param report_category: Used to filter reports with a specific type.
        :param target_id: Get reports for a specific player by passing their player ID.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param sort: Sorted by the greatest timestamps.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/reports", build_params(self.get_reports, locals()))

    def get_revives(self, attack_filters: list[AttackFiltersOptions] = None, striptags: bool = True, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's detailed revives.
        Requires faction API access permissions.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/revives", build_params(self.get_revives, locals()))

    def get_revivesfull(self, attack_filters: list[AttackFiltersOptions] = None, striptags: bool = True, limit: int = 1000, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's simplified revives.
        Requires faction API access permissions.
        API key (Limited).
        :param attack_filters: It's possible to use this query parameter to only get incoming or outgoing attacks / revives. If not specified, this selection will return both incoming and outgoing attacks / revives. It's also possible to combine this with 'idFilter'. This filter allows using from/to to filter by ids instead of timestamps.
        :param striptags: Determines if fields include HTML or not ('Hospitalized by user' vs 'Hospitalized by user').
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/revivesFull", build_params(self.get_revivesfull, locals()))

    def get_search(self, faction_name: str = None, filters: list[str] = None, limit: int = 20, offset: int = 0, timestamp: int = None, comment: str = None) -> dict: #1F4A9 endpoint
        """
        Search factions by name or other criteria.
        API key (Public).
        This selection is standalone and cannot be used together with other selections.
        :param faction_name: Name to search for.
        :param filters: A filtering query parameter allowing a comma-separated list of filters.
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/search", build_params(self.get_search, locals()))

    def get_stats(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's challenges stats.
        Requires faction API access permissions.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/stats", build_params(self.get_stats, locals()))

    def get_territory(self, faction_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get a list of territories for your faction or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/territory", build_params(self.get_territory, locals()))

    def get_territoryownership(self, limit: int = 20, offset: int = 0, timestamp: int = None, comment: str = None) -> dict:
        """
        Get a list territory ownership.
        API key (Public).
        :param limit: Number of results to return.
        :param offset: Number of rows to skip before returning results.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/territoryownership", build_params(self.get_territoryownership, locals()))

    def get_territorywars(self, faction_id: int = None, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get the territory wars history for your faction or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/territorywars", build_params(self.get_territorywars, locals()))

    def get_territorywarreport(self, territory_war_id: int, timestamp: int = None, comment: str = None) -> dict:
        """
        Get territory war details.
        API key (Public).
        :param territory_war_id: Territory war id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/territorywarreport", build_params(self.get_territorywarreport, locals()))

    def get_upgrades(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get your faction's upgrades.
        Requires faction API access permissions.
        API key (Minimal).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/upgrades", build_params(self.get_upgrades, locals()))

    def get_warfare(self, warfare_category: FacWarfareCatOptions, limit: int = 100, sort: SortOptions = "DESC", to: int = None, from_: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get faction warfare.
        API key (Public).
        The response depends on the selected category.
        :param warfare_category: Warefare categories.
        :param limit: Number of results to return.
        :param sort: Sorted by the greatest timestamps.
        :param to: Timestamp that sets the upper limit for the data returned. Data returned will be up to and including this time.
        :param from_: Timestamp that sets the lower limit for the data returned. Data returned will be after this time.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/warfare", build_params(self.get_warfare, locals()))

    def get_wars(self, faction_id: int = None, timestamp: int = None, comment: str = None) -> dict:
        """
        Get the wars and pacts details for your faction or a specific faction.
        API key (Public).
        :param faction_id: Faction id.
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/wars", build_params(self.get_wars, locals()))

    def get_lookup(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get all available faction selections.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/lookup", build_params(self.get_lookup, locals()))

    def get_timestamp(self, timestamp: int = None, comment: str = None) -> dict:
        """
        Get current server time.
        API key (Public).
        :param timestamp: Timestamp to bypass cache.
        :param comment: Comment for your tool/service/bot/website to be visible in the logs.
        :return: API response data.
        :rtype: dict
        """
        return self.api.request("/faction/timestamp", build_params(self.get_timestamp, locals()))


"""
The MIT License (MIT)

Copyright (c) 2023-Present cxdzc

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

import requests

from . import __version__
from .endpoints.faction import Faction
from .endpoints.forum import Forum
from .endpoints.key import Key
from .endpoints.market import Market
from .endpoints.property import Property
from .endpoints.racing import Racing
from .endpoints.torn import Torn
from .endpoints.user import User
from .errors import TornAPIErrorHandler

class TornAPIWrapper:
    """
    A Python wrapper for the Torn City API, providing access to Torn City data.
    """
    BASE_URL = "https://api.torn.com/v2"
    api_error_handler: TornAPIErrorHandler = TornAPIErrorHandler()

    def __init__(self, api_key: str, request_timeout: float | tuple[float, float] = 10):
        self.api_key = api_key
        self.request_timeout = request_timeout
        self.session = requests.Session()
        self.session.headers.update({"Authorization": f"ApiKey {self.api_key}", "User-Agent": f"TornAPIWrapper/{__version__}", "Accept": "application/json"})

        self.user: User = User(self)
        self.faction: Faction = Faction(self)
        self.market: Market = Market(self)
        self.forum: Forum = Forum(self)
        self.racing: Racing = Racing(self)
        self.property: Property = Property(self)
        self.key: Key = Key(self)
        self.torn: Torn = Torn(self)

    def request(self, endpoint: str, params: dict | None) -> dict:
        """
        Sends a request to the Torn API and returns the JSON response.
        :param endpoint: Torn API endpoint path.
        :param params: Query parameters to include in the request.
        :return: API response data.
        :rtype: dict
        """
        api_request = self.session.get(url=f"{self.BASE_URL}{endpoint}", params=params, timeout=self.request_timeout)
        api_request.raise_for_status()
        api_request_json = api_request.json()
        api_request_error = api_request_json.get("error")
        if api_request_error:
            self.api_error_handler.raise_code(api_request_error["code"])
        return api_request_json